\\ Copyright 2020, 2021, 2022, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ This is various tests which are either a bit slow or which depend on OEIS
\\ data available, or both.

default(recover,0)
default(strictargs,1)
default(parisizemax,1500*10^6);
read("../interv.gp");
read("../interv-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");

option_verbose=0;
option_progress = 1;

phi=quadgen(5);
log2(x) = log(x)/log(2);



\\-----------------------------------------------------------------------------
\\ A317335 yet another paperfolding  turn 1=left, -2=right
\\ a(4*n-3) = 1, a(4*n-1) = -2, a(2*n) = a(n)

A317335(n) = n>=1 || error(); \
  if(bittest(n,valuation(n,2)+1), -2, 1);
OEIS_check_func("A317335",,['offset,1]);

\\ A317336 yet another paperfolding  next turn -7=left, -10=right
\\ but exception at n=1
A317336(n) =
{
  \\ n=1 which decrements to 0 is treated as the "1 bit" case
  n>=1 || error(); 
  if(!(n--) || bittest(n,valuation(n,2)+1), -7, -10);
}
OEIS_check_func("A317336",,['offset,1]);

\\ which is where Liouville contfrac term 1
A317331(n) = n>=1 || error(); \
  n<<2 - bittest(n,valuation(n,2)+1);
OEIS_check_func("A317331",,['offset,1]);

\\ which is where Liouville contfrac term 8
\\ 8n and +1 or -2
\\ 9, 17, 22, 33, 41
A317332(n) = n>=1 || error(); 8*n + A317335(n);
OEIS_check_func("A317332",,['offset,1]);
is_A317332(n) =
{
  (n+=2)>=8 &&
  bitand(n,7) == 3*!bittest(n,valuation(n>>3,2)+4);
}
OEIS_check_predfunc("is_A317332");

\\ which is where Liouville contfrac term 9
\\ 8n and -7 or -10, exception n=1
\\   1, 6, 14, 25, 30
A317333(n) = n>=1 || error(); 8*n + A317336(n);
OEIS_check_func("A317336",,['offset,1]);
is_A317333(n) =
{
   n==1 || ((n+=2)>=8
           && bitand(n,7) == 3*bittest(n,valuation(n>>3,2)+4));
}
OEIS_check_predfunc("is_A317333");


\\-----------------------------------------------------------------------------
\\ Liouville Contfrac
\\ A012245 decimal digits, charfunc of factorials
\\ A058304 contfrac terms
\\ 0, 9, 11, 99,  1, 10, 9, 999999999999,    1,
\\    8, 10,  1, 99, 11, 9, 999...999,       1,
\\    8, 11, 99,  1, 10, 8, 1,    999999999999,
\\    9, 10,  1, 99, 11, 9

{
  my(term(n) = 1/10^(n!));
  my(limit=7,
     lo=sum(i=1,limit, term(i)),
     hi=lo + term(limit),
     t=[lo,hi]);
  print(interv_contfrac(t,12));
  if(0,
     \\ Engel is some powers of 10
     print(interv_Engel_terms(t));
     quit);
  if(1,
     \\ Egyptian is simply the 1/10^(n!)
     print(interv_Egyptian_terms(t));
     quit);

  \\ decimal digits
  OEIS_check_vecfunc("A012245",
                     (len)->interv_frac_digits(t,,len+1)[^1]);
  \\ continued fraction terms
  OEIS_check_vecfunc("A058304",
                     (len)->interv_frac_digits(t,,len+1)[^1]);

  \\ a(n) = 1 iff n in A317331     4n - BitAbove
  \\ a(n) = 8 iff n in A317332     8n +1 or -2
  \\ a(n) = 9 iff n in A317333     8n -2 or +1
  \\ a(n) = 10 iff 8n + 2 + 3*(n mod 2)   b010
  \\ a(n) = 11 iff 8n + 5 - 3*(n mod 2)   b101
  \\ a(n) = 10^((m-1)*m!)-1 iff n in 
  \\ {2^m* (4*k+1) - 1 | k >= 0}    01 00..00
  \\ {2^m* (4*k+3)     | k >= 0}    11 00..00
}




\\-----------------------------------------------------------------------------
\\ A359456 charfunc, Fiboville
\\   1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
\\
\\ vector(6,n,n++; (n-1)*n!)
\\   [2, 12, 72, 480, 3600, 30240
\\ m=3; 10^((m-1)*m!)-1
\\ logint(999999999999999999+1,10)
\\ 999999999999999999 == 10^18-1

\\ A359458 =  0, 2, 18, 180, 2640, 59280, 2096640,
\\ eset         [2, 18, 180, 2640, 59280]

{
  \\ A359838 continued fraction expansion of interpreted as binary
  my(want=[0, 1, 3, 3, 1, 2, 1, 262143, 3, 1, 3, 3, 1,
           1532495540865888858358347027150309183618739122183602175,
           4, 3, 1, 3, 262143, 1, 2, 1, 3, 3, 1]);
  my(e=1, s=0, width=1);
  for(n=2,9, e*=fibonacci(n); s += (width=1/2^e));
  print(s*1.0);
  my(t=[s,s+width]);
  my(got=interv_contfrac(t,#want));
  check_equal(got,want);
  if(1,
     print(interv_contfrac(t)); 
     quit);
}

{
  my(want=[0, 9, 11, 99, 1, 10, 9, 999999999999999999, 1, 8, 10, 1, 99, 11, 9]);

  my(e=1, s=0, width=1);
  for(n=2,9,
     e*=fibonacci(n); s += (width=1/10^e));
  print(s*1.0);
  my(t=[s,s+width]);
  my(got=interv_contfrac(t,#want));
  check_equal(got,want);

  my(got=[0, 9, 11, 99, 1, 10, 9, 999999999999999999, 1, 8, 10, 1, 99, 11, 9]);
  check_equal(got,want[1..#got]);

  my(epow10(n) = my(e=logint(n,10));
     if(n==10^e,e,0));
  print(got);
  got=interv_contfrac(t);
  my(eset=[]);
  for(n=1,#got,
     my(e=epow10(got[n]+1));
     if(e>1,
        print(n" 10^"e" - 1"); eset=setunion(eset,[e]);
        ,
        print(n" "got[n])));
  print("eset ",eset);
  quit;
}

\\-----------------------------------------------------------------------------
\\ A052119  contfrac 0; 1,2,3,...
\\   0.697774657964007982

\\ A060997  contfrac 1; 2,3,4,...
\\   1.433127426722311758

\\ A021085  decimal expansion of 1/81
\\   0.1234567901234567901234
\\   1.234567891011121314151
{
  if(0,
     my(t=interv_frac_fromdigits(concat(vector(120,i,digits(i-1)))));
     print(t*1.0);
     quit);
}
{
  my(t=interv_from_contfrac(vector(20,i,i-1)));
  print(t*1.0);
  quit;
}

\\-----------------------------------------------------------------------------
\\ A329356  Kolakoski bits

{
  my(k2=OEIS_data("A000002"),
     kbits=apply(t->2-t,k2));
  my(a(n) = fromdigits(kbits[1..n],2));
  OEIS_check_func("A329356",a,['offset,0]);

  \\ as a constant
  if(0,
     \\ kbits [1,0,0,1,1,0,1,0,0,1,0,0,1,1,0,1,1,0,0,1]
     \\   A000002  1, 2, 2, 1, 1, 2, 1, 2, 2, 1, 2, 2, 1, 1,
     \\            1221121221
     \\   A118270  0.110010110
     \\ .60274640361026036187981879218197
     
     \\ 1 - 0.7945071927794792762 / 2

     my(t=interv_frac_fromdigits(kbits,2,0));
     \\ my(t=interv_frac_fromdigits(apply(b->1-b,kbits),2,0));
     print(kbits[1..20]);
     print(t*1.0);
     print(interv_frac_digits(t));
     print(fromdigits(interv_frac_digits(t)));
     quit);
}


\\-----------------------------------------------------------------------------
\\ Newton  1/4pi = 4 arctan(1/5) - arctan(1/239)

{
  my(width_digits(t) = logint(floor(1/abs(t[1]-t[2])),10));
  my(t5=interv_atan_Taylor([1/5,1/5], 120));
  my(t239=interv_atan_Taylor([1/239,1/239], 36));
  my(t = 4*interv_add(t5*4, -t239));
  \\ print("t5   ",width_digits(t5));
  \\ print("t239 ",width_digits(t239));
  \\ print("t ",width_digits(t)"  ",t*1.0);
  \\ print(t*1.0);

  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1000],
     t_Pi=interv_frac_fromdigits(Pi_digits));
  \\ print("t_Pi ",width_digits(t_Pi)"  ",t_Pi*1.0);
  check_equal(interv_contains(t, t_Pi), 1);
}

\\-----------------------------------------------------------------------------
\\ A072398, A072399  num/den of best contfrac den <= 10^n
\\    3, 22, 22, 355, 355, 312689, 1146408,
\\   ~/OEIS/b072398.txt

{
  \\ A001203 pi contfrac
  my(v=OEIS_data("A001203",'bfile));
  my(m=contfracpnqn(v,2000),
     len=#m);

  \\ A002486 pi contfrac convergents denominators
  \\ print(m[2,][1..20]); quit;

  my(want_nums=OEIS_data("A072398"));
  my(want_dens=OEIS_data("A072399"));
  \\ my(want_nums=OEIS_data("A072398",'bfile));
  \\ my(want_dens=OEIS_data("A072399",'bfile));

  my(den_to_n(d) = if(d==1,0, logint(d-1,10)+1));
  check_equal(den_to_n(10^5), 5);
  check_equal(den_to_n(10^5-1), 5);
  check_equal(den_to_n(10^4+1), 5);
  check_equal(den_to_n(10^4), 4);
  my(hi=den_to_n(m[2,len]));
  print("hi = "hi);

  my(nums=vector(hi),
     dens=vector(hi));
  for(i=1,len,
     \\ print(m[1,i]," / ",m[2,i]);
     my(n=den_to_n(m[2,i]));
     n>=1 || next;
     nums[n] = m[1,i];
     dens[n] = m[2,i]);
  for(n=1,hi,
     if(nums[n]==0,
        nums[n]=nums[n-1];
        dens[n]=dens[n-1]));
  nums=nums[^hi];
  dens=dens[^hi];
  nums=concat(3,nums);
  dens=concat(1,dens);
  print("nums len ",#nums," dens len ",#dens);

  \\ nums=nums[1..10];
  \\ want_nums=want_nums[1..10];
  \\ dens=dens[1..10];
  \\ want_dens=want_dens[1..10];
  check_equal(want_nums, nums[1..#want_nums], "nums");
  check_equal(want_dens, dens[1..#want_dens], "dens");

  \\ default(realprecision,3*32*16);
  \\ want_nums = [3,22,22,355,355,312689,1146408,5419351,245850922,2549491779,21053343141,21053343141,1783366216531,8958937768937,139755218526789,1564184217101490,1564184217101490,43229178266965627,2629158953199395797,23145244623808076139,156758394460257741379,1320873730600286763652,23618968756344904004357,217854213729505283093821,992153446160946225924373,992153446160946225924373,79759425309091418545365116,1990017018942641678730430408,7482503677362179149875455309,229490032580876524402362417848,2251995320763534810803101877034,24312968463237129870029395811678,43110693640855860975252218139695,2678378249018681779230444098144751,25379927325216193111226340605165592,196365028326003268372222240705032706,196365028326003268372222240705032706,19943740338751939836336170601863402455,158960827625037508885572698092792121522,576208454512220219301654502806283311429,18220074860778079692526735102143388669547];
  \\ check_equal(want_nums - nums[1..#want_nums], "nums from bestappr()");
}


\\-----------------------------------------------------------------------------
\\ A357715  sqrt(16 + 32/sqrt(5))
\\   x = sqrt(16 + 32/sqrt(5))
\\   x^2 = 16 + 32/sqrt(5)
\\   x^2 - 16 = 32/sqrt(5)
\\   (x^2 - 16)^2 - 32^2/5 = 0
\\
\\ W = sqrt(2 - 2 / sqrt(5))
\\ H = sqrt(2 + 2 / sqrt(5))
\\ (W/2)^2 + (H/2)^2   \\ 1
\\ H/W   \\ phi
\\ 2*W+2*H = 5.5055...
\\ (4 / sqrt(5)) * sqrt(5 + 2 * sqrt(5))
\\ sqrt(5 + 2 * sqrt(5)) / (sqrt(5) / 4)
\\ 4 * sqrt(1 + 2 / sqrt(5)) 
\\ from A019952 = sqrt(25 + 10*sqrt(5))/5

my(c=1600+640*quadgen(20)); \
A357715_vector(len) = digits(sqrtint(floor(c*100^(len-2))));
{
  my(p(x) =  (x^2 - 16)^2 - 32^2/5);
  my(want=[5, 5, 0, 5, 5, 2, 7, 6, 8, 1, 8, 8, 4, 6, 9, 4, 1, 5, 2, 8, 2, 8, 8, 3, 8, 3, 2, 7, 6, 4, 3, 5, 5, 0, 7, 1, 8, 1, 0, 3, 5, 9, 7, 3, 4, 4, 0, 3, 2, 6, 3, 4, 6, 5, 3, 4, 6, 2, 7, 0, 3, 0, 6, 2, 4, 7, 6, 3, 8, 0, 7, 7, 5, 0, 6, 8, 6, 9, 1, 9, 4, 0, 2, 6, 3, 8, 1, 1, 9, 7, 2, 4, 4, 0, 2, 8, 0]);
  my(t=interv_frac_fromdigits(want));
  check_equal(p(t[1]) < 0, 1);
  check_equal(p(t[2]) > 0, 1);
  check_equal(A357715_vector(#want), want,
              "A357715_vector() vs DATA");
  check_equal(vector(100,len,len--; #A357715_vector(len)),
              vector(100,len,len--; len));
  check_equal(vector(500,len,len--; A357715_vector(len)),
              vector(500,len,len--; A357715_vector(len+1)[1..len]));

  my(t=interv_frac_fromdigits(A357715_vector(1000)));
  check_equal(p(t[1]) < 0, 1);
  check_equal(p(t[2]) > 0, 1);

  my(v=digits(5 5055276818846941));
  check_equal(v, want[1..#v]);

  my(c = 1600 + 640*quadgen(4*5));
  my(a_vector(len) = digits(sqrtint(floor(c*100^(len-2)))));
  check_equal(vector(40,len,len--; #a_vector(len)),
              vector(40,len,len--; len));
  check_equal(vector(40,len,len--; a_vector(len)),
              vector(40,len,len--; want[1..len]));
  \\ print(c);
  \\ print(a_vector(0));
  \\ print(a_vector(1));
  \\ print(a_vector(10));
  \\ quit;

  \\ for(p=1,500,
  \\    default(realprecision, p+8); x = sqrt
  \\              (16 + 32 / sqrt(5));
  \\
  \\              \\ Display the constant as an
  \\              example:
  \\
  \\              print(concat(vecextract(Vec(Str(x)),
  \\              Str("1.."p+1))));

  if(0,
     \\ sqrt(16 + 32/sqrt(5))
     \\ 5.505527
     my(make_vec(e) =
        default(realprecision,e);
        digits(floor(sqrt(16 + 32/sqrt(5)) * 10^e)));
     my(want=a_vector(1000));
     check_equal(make_vec(6),
                 [5, 5,0,5,5,2,7]);
     for(e=10,#want-1,
        my(got);
        iferr(got=make_vec(e),err,
              print("e="e" error");
              next);
        if(got!=want[1..e+1],
           print("bad at e="e);
           print(got[max(1,#got-5)..#got]);
           print(want[#got-5..#got]);
           print("prec ",default(realprecision),
                 " words ",default(realbitprecision)/32);
          ));
     quit);
  if(0,
     \\ sqrt(16 + 32/sqrt(5))
     my(p=5);
     my(scale=1000000,
        t=[5,5]*scale^2);
     print("5 = ",t*1.0);
     t = interv_sqrtint(t) / scale;
     print("sqrt(5) = ",t*1.0);
     t = interv_recip(t);
     print("1/sqrt(5) = ",t*1.0);
     t *= 32;
     print("32/sqrt(5) = ",t*1.0);
     t = interv_add_constant(t, 16);
     t = interv_sqrtint(t*scale^2) / scale;
     print(t[1]*1.0);
     print(t[2]*1.0);
     quit);
}


\\-----------------------------------------------------------------------------
\\ Pi
\\ A006784  Engel expansion pi
{
  my(digs=OEIS_data("A000796",'bfile)[1..300],
     t=interv_frac_fromdigits(digs));

  \\ Engel
  OEIS_check_vecfunc("A006784",
    len->interv_Engel_terms(t,len));
}


\\-----------------------------------------------------------------------------
\\ A165420  2^(2^(n-2-(n>2)))
\\   Engel expansion of KM + 1

A165420(n) = 1<<(1<<(n-2-(n>2)));
{
  my(a=A165420);
  OEIS_check_func("A165420",,['offset,1]);

  \\ code by G. C. Greubel in A165420
  my(b(n) = if(n==1, 1, if(n==2, 2,
               prod(j=1, n-1, self()(j)))));
  check_equal(vector(8,n,b(n)),
              vector(8,n,a(n)));
}

\\-----------------------------------------------------------------------------
\\ Kempner-Mahler

{
  my(L = 10,
     lo = sum(i=0,L, 1/2^(2^i)),
     hi = lo + 2/2^(2^(L+1)),
     t=[lo,hi]);
  check_equal(1/2^(2^0), 1/2, "KM first term");
  check_equal(1/2^(2^1), 1/4, "KM second term");
  \\ print(t*1.0);

  \\ decimal 0.81642150902189314370....
  OEIS_check_vecfunc("A007404",
    (len)->interv_frac_digits(10*t,,len));

  \\ contfrac
  OEIS_check_vecfunc("A007400",
    (len)->interv_contfrac(t,len));

  \\ Engel
  \\ A165420 = a(1) = 1, a(2) = 2, then product of previous
  \\   1,2,2,4,16,256,65536,4294967296
  my(got=interv_Engel_terms(interv_add_constant(t,1)));
  check_equal(#got,12);
  check_equal(vector(#got,n, A165420(n)), got);

  my(got=interv_Engel_terms(t));
  check_equal(vector(#got,n,n++; A165420(n)), got);
}

\\-----------------------------------------------------------------------------
\\ Primes Constant, primes charfunc as binary

{
  my(v=vector(1000,n, isprime(n)),
     t=interv_frac_fromdigits(v,2,0));

  \\ decimal 0.414682509851111660
  my(want=OEIS_data("A051006"));
  my(got=interv_frac_digits(t,,#want+1)[^1]);
  check_equal(got,want, "A051006 prime constant decimal");

  \\ Egyptian
  my(want=OEIS_data("A036680"));
  my(got=interv_Egyptian_terms(t,#want));
  check_equal(got,want, "A051006 prime constant Egyptian");
  \\ example in A051006
  my(got=digits(011010100010100010100010000010100));
  check_equal(v[1..#got+1], concat(0,got));
  \\ 1/2^3 + 1/2^13 + 1/2^226.
  \\ 1/3 + 1/13 + 1/226 + 1/757098 + 1/1493980747140.
  \\ 0.41468250

  if(0,
     print(interv_Engel_terms(t,12));
     \\ not in OEIS: 3, 5, 5, 10, 85, 100, 3325
     quit);
}


\\-----------------------------------------------------------------------------
\\ A357469
\\   real root of x^3 - x^2 + x - 2
{
  my(f(x) = x^3 - x^2 + x - 2);
  my(want=OEIS_data("A357469"),
     t=interv_solve_bisection([1,2],f,1000),
     tf=interv_subst(f(x),x,t));
  check_equal(interv_contains_0(tf), 1,
              "A357469 tf crosses 0");
  my(got = interv_frac_digits(t,,#want));
  check_equal(got, want);

  check_equal(f(x-1), x^3 - 4*x^2 + 6*x - 5);
}

\\-----------------------------------------------------------------------------
\\ A197032 Philo line 2,1
\\   real root of x^3 - 4*x^2 + 6*x - 5
\\   data fixed Nov 2022

{
  my(f(x) = x^3 - 4*x ^2 + 6*x - 5);
  my(want=[2, 3, 5, 3, 2, 0, 9, 9, 6, 4, 1, 9, 9, 3, 2, 4, 4, 2, 9, 4, 8, 3, 1, 0, 1, 3, 3, 2, 5, 7, 7, 3, 8, 8, 4, 5, 7, 2, 7, 0, 7, 0, 5, 6, 1, 3, 8, 5, 6, 8, 4, 6, 8, 2, 6, 8, 0, 6, 6, 9, 3, 0, 4, 2, 6, 5, 1, 5, 1, 8, 9, 7, 2, 3, 2, 2, 0, 9, 2, 0, 8, 5, 9, 1, 6, 5, 8, 0, 3],
     t=interv_solve_bisection([2,3],f,1000),
     tf=interv_subst(f(x),x,t));
  check_equal(interv_contains_0(tf), 1,
              "tf crosses 0");
  my(got = interv_frac_digits(t,,#want));
  check_equal(got, want);
}

\\-----------------------------------------------------------------------------
\\ A357969  sum j!/primorial(j)
\\ 2.240053652689050110257

{
  my(term(n) = n!/vecprod(primes(n)));
  my(limit=200,
     s=sum(i=0,limit,term(i)),
     t=[s,s+term(limit)]);
  my(want=OEIS_data("A357969"));
  my(got=interv_frac_digits(t,,#want));
  check_equal(got,want);

  my(e=2 240053652689050110257);
  check_equal(digits(e), want[1..#digits(e)]);
}


\\-----------------------------------------------------------------------------
\\ A001466 Pi-3 Egyptian Fraction
\\    8, 61, 5020, 128541455, 162924332716605980, 28783052231699298507846309644849796
\\ vs 8, 61, 5020, 128541457, 7862753935607729
\\
\\ A182257 Pi  Egyptian Fractions
\\   1, 1, 1, 8, 61, 5020, 128541455,
\\   using run of 1s for the integer part
\\
\\ A224230 Pi  Egyptian Fractions
\\   3, 8, 61, 5020, 128541455,
\\   using 3 for integer part

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1000],
     t_Pi=interv_frac_fromdigits(Pi_digits),
     t_Pi_sub3 = interv_sub_constant(t_Pi,3));

  \\ Pi-3
  my(want=OEIS_data("A001466"));
  my(got=interv_Egyptian_terms(t_Pi_sub3,#want));
  check_equal(got,want, "A001466 Pi-3 Egyptian");

  \\ 7, 790, 749896, 1270073831726
  my(want=OEIS_data("A014013"));
  my(got=interv_Egyptian_terms_alternating(t_Pi_sub3,#want));
  check_equal(got,want, "A014013 Pi-3 Egyptian alternating");



  \\ Pi
  my(want=OEIS_data("A182257"));
  my(got=interv_Egyptian_terms(t_Pi,#want));
  check_equal(got,want, "Pi Egyptian");
}

\\-----------------------------------------------------------------------------
\\ A336810  contfrac sum 1/(k!)!
\\

A336810_sum_interval(k_max) =
{
  my(s=0,t);
  for(k=0,k_max,
     t = 1/(k!)!;
     s += t);
  [s, s+t];
}
{
  my(t=A336810_sum_interval(6));
  my(want=OEIS_data("A336810"));
  check_equal(interv_contfrac(t,#want), want,
              Str("A336810 new data length ",#want));

  my(want=OEIS_data("A336810"));
  check_equal(interv_contfrac(t,#want), want,
              Str("A336810 data length ",#want));

  if(0,
     my(want=OEIS_data("A336810",'bfile));
     my(a(n) = want[n+1]);
     check_equal(#digits(a(11)), 152);
     check_equal(#digits(a(21)), 1349);
     check_equal(#digits(a(41)), 12981);

     check_equal(interv_contfrac(t,#want), want,
              Str("A336810 bfile length ",#want)));

  if(0,
     my(m=contfracpnqn(want,10));
     print(m[1,]);
     print(m[2,]);
     \\ not in OEIS: 2, 3, 5, 898, 1801, 2155531580867214949497
     \\ not in OEIS: 1, 1, 2, 359, 720, 861733891296165887639
     quit);
}



\\-----------------------------------------------------------------------------
\\ A004601 Pi in Binary
\\ A180433 bitwise complement
\\ suminf(k=1,1/2^prime(k))

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..3100],
     t_Pi=interv_frac_fromdigits(Pi_digits));

  my(A004601_vector(len) =
     interv_frac_digits(t_Pi/2,2,len));
  OEIS_check_vecfunc("A004601_vector",A004601_vector);

  my(A180433_vecctor(len) =
     apply(t->1-t, interv_frac_digits(t_Pi/2,2,len)));
  OEIS_check_vecfunc("A180433_vecctor",A180433_vecctor);
  if(0, OEIS_check_vecfunc("A180433_vecctor",A180433_vecctor,'bfile));
  check_equal(#A180433_vecctor(10),10);
  check_equal(#A180433_vecctor(10000),10000);
  if(0,
     my(v=A180433_vecctor(10000));
     system("rm -f      /tmp/b.txt");
     for(n=1,#v, write("/tmp/b.txt",n," ",v[n]));
     system("ls -l      /tmp/b.txt");
     quit);
}



\\-----------------------------------------------------------------------------
\\ sqrt(27)
\\   A248254  decimal
\\   A248254  Egyptian
\\   Engel only A028257 of sqrt(3)

{
  my(sqrt_digits=OEIS_data("A010482"),
     t_sqrt=interv_frac_fromdigits(sqrt_digits),
     t_sqrt_exact=vector(2,i, 3*quadgen(4*3)));
  check_equal(interv_contains(t_sqrt, t_sqrt_exact), 1);

  \\ Egyptian fractions = 5, 6, 34, 13516
  \\   so 5 + 1/6 + 1/24 + ...
  my(want=OEIS_data("A248254"),
     got=interv_Egyptian_terms(interv_sub_constant(t_sqrt_exact,4), #want));
  got[1] += 4;
  check_equal(got,want, "sqrt(27) Egyptian");
}


\\-----------------------------------------------------------------------------
\\ Engel expansion sqrt(157)

\\ Engel
\\ not in OEIS: 1, 4, 84, 148, 185, 2503, 6978, 23592
\\ Egyptian
\\ not in OEIS: 1, 4, 334, 417252, 203698861781

{
  \\ my(sqrt_digits=OEIS_data("A178310"),
  \\    t_sqrt=interv_frac_fromdigits(sqrt_digits),
  \\    want=OEIS_data(A006784),
  \\    got=interv_Egyptian_terms(t_sqrt,#want));
  \\ check_equal(got,want);
}


\\-----------------------------------------------------------------------------
\\ Pi Continued Fraction

\\ return number of starting elements u[i]==v[i]
vector_match_length(u,v) =
{
  my(len=min(#u,#v));
  for(i=1,len, if(u[i]!=v[i],return(i-1)));
  len;
}

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..1100],
     t_Pi=interv_frac_fromdigits(Pi_digits));
  my(l=interv_contfrac(t_Pi));
  print("contfrac length ",#l);

  \\ matrix rows nums and dens
  my(m_convs=contfracpnqn(l,#l+1));
  if(0,
     for(i=1,min(3,#m_convs),
        print(m_convs[1,i]" / "m_convs[2,i])));

  my(A002485_convergent_num(n) =
     n>=0 || error();
     if(n<=1,n, m_convs[1,n-1]));
  my(A002486_convergent_den(n) =
     n>=0 || error(); 
     if(n<=1,1-n, m_convs[2,n-1]));
  OEIS_check_func("A002485_convergent_num",A002485_convergent_num);
  OEIS_check_func("A002486_convergent_den",A002486_convergent_den);

  OEIS_check_vecfunc("A002485",
    len->concat([0,1],m_convs[1,][1..len-2]));
  OEIS_check_vecfunc("A002486",
    len->concat([1,0],m_convs[2,][1..len-2]));

  my(convergent(n) = 
     A002485_convergent_num(n) / A002486_convergent_den(n));
  check_equal(convergent(5), 355/113);

  \\ num_match() returns how many digits of pi are correct in the initial
  \\ digits of fraction f.
  \\
  \\ "estimate" is an estimated number of digits which is used for where to
  \\ look for a digit of f disagreeing with pi.  A good estimate is a
  \\ speedup but the result is the same no matter what estimate.
  \\ 
  my(num_match(f,estimate=32) =
     while(1,
           my(v=digits(floor(f*10^estimate)));
           #v < #Pi_digits || error();
           my(got=vector_match_length(Pi_digits,v));
           if(got<#v, return(got));
           estimate=ceil(estimate*1.1+30)));
  check_equal(num_match(22/7), 3,
             "22/7 = 3.142");

   \\ A356665 num digits of convergents
  my(want=[1, 3, 5, 7, 10, 10, 10, 10, 12, 11, 13, 13, 15, 16, 16, 17, 18, 18, 19, 20, 22, 24, 25, 25, 26, 28, 30, 31, 31, 33, 34, 35, 38, 40, 41, 41, 42, 43, 45, 46, 46, 47, 48, 50, 51, 52, 52, 54, 55, 56, 56, 57, 57, 59, 60, 60, 61, 61, 62, 61, 63, 65, 64]);
  check_equal(#want,63);

  my(A356665(n) = 
     n>=2 || error();
     num_match(convergent(n), ceil(1.05*n+20)));
  check_equal(vector(#want,n,n++; A356665(n)), \\ OFFSET=2
              want,
              "A356665 num digits vs DATA");
  \\ print(vector(#want,n,A356665(n)));

  if(0,
        \\ while(f>=1, f/=10);
        \\ for(i=1,#Pi_digits,
        \\    \\ print("at f="f" "f*1.0);
        \\    f*=10;
        \\    my(d=floor(f));
        \\    f-=d;
        \\    if(d!=Pi_digits[i],return(i-1));
        \\ error("match more than Pi_digits"));

     print("A356665 terms");
     my(record=-oo,
        rlo=oo);
     for(n=1,#l,
        my(got=A356665(n));
        my(numlen=logint(convergent_num(n),10)+1);
        my(denlen=logint(convergent_den(n),10)+1);
        my(tot=numlen+denlen,
           diff = got - tot);
        print("  "n" "got" diff ",diff,
              if(diff>record,
                 record=diff;"  HIGH",
                 ""),
              if(diff<rlo,
                 rlo=diff;"  LOW",
                 "")));
     quit);
}



\\-----------------------------------------------------------------------------
\\ https://oeis.org/draft/A265296
\\ Sum_{n>=1} (c(2*n) - c(2*n-1))
\\   convergents to sqrt(3)
{
  if(0,
     my(want=OEIS_data("A265296"),
        bits=1000,
        lo=sqrtint(3<<(2*bits)) / 2^bits,
        hi=lo+1/2^bits);
     check_equal(lo^2<3, 1);
     check_equal(hi^2>3, 1);
     my(t=[lo,hi],
        cfrac=interv_contfrac(t),
        convmat=contfracpnqn(cfrac,#cfrac),
        convs=vector(#convmat,i, convmat[1,i]/convmat[2,i]));
  \\ for(i=1,8,print(convs[2*i]*1.0));
  \\ for(i=1,8,print((convs[2*i]-convs[2*i-1])*1.0));
     my(diffs=vector(#convs\2,i, convs[2*i]-convs[2*i-1]));
     my(altsum=vecsum(diffs));
     print("final diff ",diffs[#diffs]*1.0);
     print(floor(altsum*10^100));
     quit);
}

\\-----------------------------------------------------------------------------
\\ A356032  decimals x^4 + x - 1

{
  my(want=[7, 2, 4, 4, 9, 1, 9, 5, 9, 0, 0, 0, 5, 1, 5, 6, 1, 1, 5, 8, 8, 3, 7, 2, 2, 8, 2, 1, 8, 7, 0, 3, 6, 5, 6, 5, 7, 8, 6, 4, 9, 4, 4, 8, 1, 3, 5, 0, 0, 1, 1, 0, 1, 7, 2, 7, 0, 3, 9, 8, 0, 2, 8, 4, 3, 7, 4, 5, 2, 9, 0, 6, 4, 7, 5, 1]);
  my(f(x) = x^4 + x - 1);
  my(t = interv_solve_bisection([1/2,1], f, 1000),
     lo= fromdigits(want)   /10^#want,
     hi=(fromdigits(want)+1)/10^#want);
  check_equal(interv_frac_digits(t,,#want+1),
              concat(0,want),
              "interv_frac_digits");
  check_equal(f(lo) < 0, 1);
  check_equal(f(hi) > 0, 1);

  my(e=digits(724491959000515611588372282187036565786494481350011017270));
  check_equal(e,want[1..#e]);

  \\ A060007 other real root (negative)
  my(t = - interv_solve_bisection([-2,-1], f, 1000),
     want=OEIS_data("A060007"));
  check_equal(interv_frac_digits(t,,#want),
              want,
              "A060007 interv_frac_digits");

  localprec(1000);
  my(rr=polroots(f(x)),
     z=rr[4]);
  check_equal(floor(real(z)*10^9),
              248126062);
  check_equal(floor(imag(z)*10^9),
              1 033982060);

  my(Ap = (3/16)*(9 + sqrt(3*283)),
     Am = (3/16)*(9 - sqrt(3*283)),
     ep = (-1 + sqrt(3) *I)/2,
     u = (Ap^(1/3) + ep*Am^(1/3))/3,
     r = (-sqrt(2)*u + sqrt(sqrt(2*u) - 2*u^2))
        / (2*sqrt(u)));
  check_equal(abs(r-rr[2]) < 1e-500, 1);

  u = (2/3)*sqrt(3)*sinh((1/3)*asinh((3/16)*sqrt(3)));
  r = (-sqrt(2)*u + sqrt(sqrt(2*u) - 2*u^2))
     / (2*sqrt(u));
  check_equal(abs(r-rr[2]) < 1e-500, 1);

  r = (-sqrt(2)*u + sqrt(-sqrt(2*u) - 2*u^2))
     / (2*sqrt(u));
  printf("real r  %8.f\n",real(r));
  printf("real r4 %8.f\n",real(rr[4]));
  printf("%8.f\n",imag(r));
  printf("%8.f\n",imag(rr[4]));
  check_equal(abs(real(r) - (-real(rr[4]))) < 1e-500, 1);
  check_equal(abs(imag(r) - imag(rr[4])) < 1e-500, 1);
}


\\-----------------------------------------------------------------------------
\\ A331692  continued fraction of sqrt(phi)
\\   1, 3, 1, 2, 11, 3, 5
\\ also a331692-contfrac-sqrt-phi--test.gp


\\-----------------------------------------------------------------------------
\\ A355575  pi Engel variant  (rejected)

make_pi_digits(len) =
{
  localprec(len+1000);
  digits(floor(Pi * 10^(len-1)));
}

\\ Pi - 3 = 3/(22)
\\        + 1/(22*9)
\\        + 4/(22*9*114)
\\        + 1/(22*9*114*35) + ...
{
  \\ my(Pi_digits=OEIS_data("A000796",'bfile));
  my(bits=1000,
     Pi_digits=make_pi_digits(bits));
 \\ Pi_digits = Pi_digits[1..300];
  print("Pi_digits length ",#Pi_digits);
  my(t_Pi=interv_frac_fromdigits(Pi_digits),
     want=[22, 9, 114, 35, 178, 511, 127, 570, 1050, 956, 2323, 8125, 29166, 41549, 76365, 231887, 1159745, 9508561, 48906603, 129211492, 366532653, 137182041, 413412878, 375119962, 989323988, 1656855777, 413670705959, 259516290562, 2392983002184, 14857501231366]);

  \\ 0 term means denominator factor 1
  if(0,
     my(nums=[1,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1],
        r=vector(21),
        a=vector(20));
     r[1]=Pi;
     for(n=1,#a,
        a[n] = floor((1 / r[n])*nums[n] + 1);
        r[n+1] = r[n]*a[n] - nums[n]);
     print(a);
     quit);

  my(debug=0);
  nums=Pi_digits;
  if(debug,
     for(i=1,#nums,
        if(nums[i]==0,
           print("nums["i"] = "nums[i]);
           break)));

  my(t=interv_sub_constant(t_Pi,3),
     limit=max(#want,40),
     ret=List([]));
  limit = oo;
  while(t[1]>0 && #ret<limit,
        my(pidig=nums[#ret+1]);

        my(d=floor(interv_div([pidig,pidig],t)));
        d=interv_add_constant(d,1);
        \\ my(d=ceil(interv_div([pidig,pidig],t)));

        if(debug,
           print("at "#ret" t="t*1.0);
           print("  pidig="pidig" d="d);
           print());
        if(d[1]!=d[2],
           if(debug, print("stopp for disagree"));
           break);
        listput(ret,d[1]);
        t=interv_sub_constant(t*d[1], pidig));
  if(debug, print("final t=",t*1.0));
  ret=Vec(ret);

  print("pseudo-Engel length ",#ret);
  check_equal(ret[1..#want], want,
              "A355575 pseudo-Engel expansion of pi");

  my(pidigit(n) = floor(Pi*10^(n-1)) % 10);
  check_equal(vector(6,n,pidigit(n)),
              [3, 1, 4, 1, 5, 9]);

  if(0,
     my(r=vector(10),
        a=vector(10));
     r[1]=Pi-3;
     for(n=1,9,
        a[n] = floor( (1/r[n])*pidigit(n) + 1);
        print("n="n" a[n] = "a[n]);
        r[n+1] = r[n]*a[n] - Pi_digits[n]);
     print(a);
     quit);
}

\\-----------------------------------------------------------------------------
\\ A356030  x^3 - x - 2
{
  my(want=[1, 5, 2, 1, 3, 7, 9, 7, 0, 6, 8, 0, 4, 5, 6, 7, 5, 6, 9, 6, 0, 4, 0, 8, 0, 8, 3, 2, 2, 5, 4, 4, 3, 8, 5, 1, 4, 4, 2, 8, 3, 8, 9, 8, 2, 8, 4, 2, 7, 9, 0, 3, 9, 0, 9, 0, 9, 0, 4, 9, 8, 0, 1, 5, 4, 2, 8, 1, 5, 6, 4, 0, 3, 4, 3, 0]);

  my(v=digits(15213797068045675696040808322544385144283898284279039090904980154281564));
  check_equal(want[1..#v],v);

  my(t = interv_solve_bisection
        ([1,2], x-> x^3 - x - 2, 1000));
  check_equal(interv_frac_digits(t,,#want),
              want);
}


\\-----------------------------------------------------------------------------
\\
{
  my(want=[5, 8, 9, 7, 5, 4, 5, 1, 2, 3, 0, 1, 4, 5, 8, 3, 8, 4, 2, 7, 8, 8, 0, 1, 7, 4, 7, 0, 9, 6, 0, 7, 1, 3, 6, 2, 4, 5, 1, 4, 3, 5, 1, 7, 3, 2, 2, 8, 3, 6, 2, 0, 7, 2, 3, 8, 8, 0, 7, 9, 3, 4, 1, 2, 3, 5, 0, 4, 8, 0, 2, 4, 1, 5, 7, 7]);

  my(v=digits(5897545123014583842788017470960713624514351732283620723880793412350480241));
  check_equal(want[1..#v],v);

  my(t = interv_solve_bisection
        ([0,1], x-> 2*x^3 + x - 1, 1000));
  check_equal(interv_frac_digits(t,,#want+1),
              concat(0,want));
}


\\-----------------------------------------------------------------------------
\\ A356585  num digits Gosper hyperfactorial n,n

\\ 1^(1^m)*2^(2^m)*3^(3^m)*...*n^(n^m)
Gosper_hyperfactorial(n,m) = prod(i=1,n, i^(i^m));

A330716(n) = Gosper_hyperfactorial(n,n);
OEIS_check_func("A330716",,['offset,0])

{
  my(want=[1, 1, 2, 16, 198, 2927, 50060, 979361, 21645853, 534381060, 14590180163, 436814197446, 14235563000269, 501817445873045, 19029286646922723, 772532087068933899, 33434018751249535666, 1536767964161539414904, 74769012084248550773909]);
  check_equal(vector(6,n,n--; #digits(A330716(n))),
              want[1..6]);

  \\ FIXME: This is not exact.
  my(log10_prec = 2^100);
  my(log10_interval(n) =
     localprec(1000);
     my(L=log(n)/log(10));
     L*=log10_prec;
     [floor(L),ceil(L)]/log10_prec);
  \\ print("log10 of 2 = ",log10_interval(2));

  my(Gosper_hyperfactorial_log10(n,m) =
     my(t=[0,0]);
     for(i=2,n,
        t = interv_add(t, log10_interval(i)*(i^m)));
     t);
  my(a(n) =
     if(n<=1, return(1));
     my(t=Gosper_hyperfactorial_log10(n,n),
        c=interv_ceil(t));
     if(c=='none,error());
     c);

  check_equal(Gosper_hyperfactorial(2,2), 16);
  check_equal(interv_ceil(Gosper_hyperfactorial_log10(2,2)), 2);
  \\ print(Gosper_hyperfactorial_log10(2,2)*1.0);

  check_equal(Gosper_hyperfactorial(3,3),
              1952152956156672, "3,3");
  check_equal(logint(Gosper_hyperfactorial(3,3),10),
              15, "logint");
  check_equal(interv_ceil(Gosper_hyperfactorial_log10(3,3)), 16);
  \\ print(Gosper_hyperfactorial_log10(2,2)*1.0);

  check_equal(vector(5,n,n--; a(n)),
              want[1..5],
              "five terms");
  check_equal(vector(#want,n,n--; a(n)),
              want,
              "DATA");
}

{
  my(want=[1, 1, 5, 51, 657, 9722, 166296, 3253365, 71905965, 1775175455, 48467529392, 1451065354742, 47289516677131, 1667001471950287, 63213921938077523, 2566296044236261518, 111065406214766719510, 5105032675471072965466, 248377281869637961805657]);
  check_equal(vector(6,n,n--; #binary(A330716(n))),
              want[1..6]);

  my(log2_prec = 2^200);
  my(log2_interval(n) =
     localprec(1000);
     my(L=log(n)/log(2));
     L*=log2_prec;
     [floor(L),ceil(L)]/log2_prec);
  \\ print("log2 of 3 = ",log2_interval(3)*1.0);

  my(Gosper_hyperfactorial_log2(n,m) =
     my(t=[0,0]);
     for(i=2,n,
        t = interv_add(t, log2_interval(i)*(i^m)));
     t);

  check_equal(#binary(16), 5);
  check_equal(Gosper_hyperfactorial(2,2),
              16, "2,2");

  check_equal(Gosper_hyperfactorial(3,3),
              1952152956156672, "3,3");
  check_equal(logint(Gosper_hyperfactorial(3,3),2),
              50, "logint");
  check_equal(interv_ceil(Gosper_hyperfactorial_log2(3,3)), 51);
  \\ print(Gosper_hyperfactorial_log2(2,2)*1.0);

  my(a(n) =
     if(n<=1, return(1));
     if(n==2, return(5));  \\ tricked by power of 2
     my(t=Gosper_hyperfactorial_log2(n,n),
        c=interv_ceil(t));
     if(c=='none,error());
     c);

  check_equal(vector(5,n,n--; a(n)),
              want[1..5],
              "binary five terms");
  check_equal(vector(#want,n,n--; a(n)),
              want,
              "binary DATA");
}





\\-----------------------------------------------------------------------------
\\ A105165 e base phi

\\ code by Chittaranjan Pardeshi in A105165
alist(len) =
{
  my(phi = quadgen(5), n=1, f=1, r=1/f, e=1+1, hi=e+r, t=0, w=phi^3);
  vector(len, i,
         w/=phi;
         while ( t+w < hi && t+w > e,
                n += 1;
                f *= n;
                r = 1/f;
                e += r;
                hi = e+r );
         if (t+w <= e, t += w; 1, 0));
}

e_interv(k) =
{
  my(lo=sum(i=0,k, 1/i!));
  [lo, lo+1/k!];
}

{
}

{
  my(t=e_interv(100),
     phi=quadgen(5));
  \\ print(t*1.0);
  foreach([["A001113",10, 1],
           ["A004593",2,  1/2],
           ["A004594",3,  1],
           ["A004595",4,  1],
           ["A004596",5,  1],
           ["A004597",6,  1],
           ["A004598",7,  1],
           ["A004599",8,  1],
           ["A004600",9,  1],
           ["A105165",phi, 1/phi^2]
          ], elem,
     my(anum,base,f); [anum,base,f] = elem;
     my(want=OEIS_data(anum));
     check_equal(interv_frac_digits(t*f,base,#want),
                 want,
                 Str("digits e base="base)));

  my(want=OEIS_data("A105165"));
  for(len=0,#want,
     alist(len) == want[1..len] || error());

  my(want=interv_frac_digits(t/phi^2,phi,1000));
  print("e base phi len ",#want);
  for(len=0,#want,
     alist(len) == want[1..len] || error());
}


\\-----------------------------------------------------------------------------
\\ Sqrt 21

{
  \\ sqrt21 as quad
  my(sqrt21=quadgen(4*21),
     t=[sqrt21,sqrt21]);
  check_equal(t[1]^2 == 21, 1, "sqrt21 quad");

  \\ Engel expansion of sqrt(21)
  if(0,
     print(interv_Engel_terms(t,10));
     \\ not in OEIS: 2,7,7,11,61,80,256,626,39928
     quit);

  \\ A248249 greedy Egyptian fractions sqrt(21)
  \\   4, 2, 13, 177, 344766
  my(want=OEIS_data("A248249"));
  my(got=interv_Egyptian_terms(interv_sub_constant(t,3),
                               #want));
  got[1] += 3;
  check_equal(got,want,
              "interv_Egyptian_terms() greedy sqrt21");

  foreach([["A010477",10]
          ], elem,
     my(anum,base); [anum,base] = elem;
     my(want=OEIS_data(anum));
     check_equal(interv_frac_digits(t,base,#want),
                 want,
                 Str("digits sqrt21 base="base)));
  \\ hexadecimal
  \\ not in OEIS: 9, 5, 2, 3, 10, 14, 4, 5, 4, 7, 10
}


\\-----------------------------------------------------------------------------
\\ A067614 Continued Fraction of sqrt(prime(n)) Second Term

A067614(n) = my(r); 2*sqrtint(prime(n),&r) \ r;
{
  OEIS_check_func("A067614",,['offset,1]);

  my(by_quad(n) = my(x=quadgen(4*prime(n)));
     x = frac(x);
     x = 1/x;
     floor(x));
  check_equal(vector(10000,n, A067614(n)),
              vector(10000,n, by_quad(n)),
              "by_quad");

  my(by_interv(n) = my(p=prime(n),
                       t=interv_solve_bisection
                         ([0,p], x->x^2 - p, 100));
     interv_contfrac(t,2)[2]);
  check_equal(vector(1000,n, A067614(n)),
              vector(1000,n, by_interv(n)),
              "by_interv");

  my(by_formula(n) = my(p=prime(n),
                        s=sqrtint(p),
                        r=p-s^2);
     floor(2*s/r));
  check_equal(vector(10000,n, A067614(n)),
              vector(10000,n, by_formula(n)),
              "by_interv");
}
A000006(n) = sqrtint(prime(n));
OEIS_check_func("A000006",,['offset,1]);

A056892(n) = my(r); sqrtint(prime(n),&r); r;
OEIS_check_func("A056892",,['offset,1]);

{
  my(by_anums(n) = my(s=A000006(n),r=A056892(n));
     floor(2*s/r));
  check_equal(vector(10000,n, A067614(n)),
              vector(10000,n, by_anums(n)),
              "by_anums");
}

\\ vector(20,n, my(r,s=sqrtint(prime(n),&r)); s\r)
\\ not in OEIS: 1, 0, 2, 0, 1, 0, 4, 1, 0, 1, 0, 6, 1, 0, 0, 1, 0, 0, 2, 1

\\ third term of contfrac
\\ vector(10,n, interv_contfrac(vector(2,i,quadgen(4*prime(n))),3)[3])
\\ not in OEIS: 2, 2, 4, 1, 6, 1, 8, 1, 3, 1


\\-----------------------------------------------------------------------------
\\ A352678 sqrt2 base phi

{
  my(want=OEIS_data("A352678"),
     t_sqrt2 = interv_solve_bisection
              ([1,2], x->x^2-2, 1000));
  my(phi=quadgen(5));
  my(got=interv_frac_digits(t_sqrt2,phi,#want));
  check_equal(got,want,
              "sqrt2 base phi");

  my(got=digits(1 01000001010010100100000001010000000001010101010101001));
  check_equal(got,want[1..#got], "example");

  \\ my code
  my(a_vector(len) = my(t=0,w=1);
     vector(len,i,
            my(tw=t+w); w/=phi;
            if(tw^2 <= 2, t=tw;1, 0));
    );
  check_equal(a_vector(#want),want);

  \\ code by Chittaranjan Pardeshi in A352679
  my(alist(len) =
     my(phi = quadgen(5), w=phi, t = 0);
     vector(len, i, w /= phi;
            if ( ( t + w )^2 <= 2, t += w ; 1, 0)));
  my(by_interv=interv_frac_digits(t_sqrt2,phi));
  print("sqrt2 by_interv ldnegth ",#by_interv);
  check_equal(by_interv,alist(#by_interv));
}

\\-----------------------------------------------------------------------------
\\ A352679 sqrt3 base phi

{
  my(want=OEIS_data("A352679"),
     prec_bits = 1000,
     t_sqrt3 = interv_solve_bisection
              ([1,2], x->x^2 - 3, prec_bits),
     t_phi = interv_solve_bisection
              ([1,2], x-> x^2-x-1, prec_bits),
     t_sqrt3_over_phi = interv_div(t_sqrt3,t_phi));
  \\ check_equal(interv_contains(t_sqrt3, sqrt(3)), 1);
  my(phi=quadgen(5));
  my(got=interv_frac_digits(t_sqrt3_over_phi,phi,#want));
  check_equal(got,want,
              "sqrt3 base phi");

  my(a_vector(len) = my(t=0,w=phi);
     vector(len,i,
            my(tw=t+w); w/=phi;
            if(tw^2 <= 3, t=tw;1, 0));
    );
  check_equal(a_vector(#want),want,
              "sqrt3 base phi a_vector()");

  \\ code by Chittaranjan Pardeshi in A352679
  my(alist(len) =
     my(phi = quadgen(5), w=phi^2, t =0);
     vector(len, i,
            w /= phi;
            if ( ( t + w )^2 <= 3,
                t += w; 1,  0)));
  my(by_interv=interv_frac_digits(t_sqrt3_over_phi,phi));
  print("sqrt3 by_interv ldnegth ",#by_interv);
  check_equal(by_interv,alist(#by_interv));
}

\\-----------------------------------------------------------------------------
\\ A352594 sqrt(phi) base phi

{
  my(want=OEIS_data("A352594"));

  \\ 1.2360679
  \\ 1.2720196
  \\ sqrt(phi)
  my(a(n)=want[n]);
  \\ print(sum(i=1,#want, 0. + a(i)/phi^(i-1))); quit;
  \\ print(0. + a(1) + a(2)/phi + a(3)/phi^2 + a(4)/phi^3 + a(5)/phi^4); quit;

  my(got=digits(1 001000100000010010001000001000100010101010100010101));
  check_equal(got,want[1..#got], "example");

  \\ my code
  my(a_vector(len) = my(t=0,w=1);
     vector(len,i,
            my(tw=t+w); w/=phi;
            if(tw^2 <= phi, t=tw;1, 0));
    );
  check_equal(a_vector(#want),want);

  my(t_sqrt_phi = interv_solve_bisection
                 ([1,2], x->x^4-x^2-1, 1000));
  \\ print(t_sqrt_phi*1.0);

  \\ code by Chittaranjan Pardeshi in A352594
  my(alist(len) =
     my(phi = quadgen(5), w=phi, t = 0);
     vector(len, i,
            w = w / phi;
            if ( ( t + w )^2 <= phi,
                t = t + w ;
                1,
                0)));
  my(by_interv=interv_frac_digits(t_sqrt_phi,phi));
  print("sqrt(phi) by_interv ldnegth ",#by_interv);
  check_equal(by_interv,alist(#by_interv));
}

\\-----------------------------------------------------------------------------
\\ A173995 Contfrac of Sum 1/Fermat primes A019434
\\ 0, 1, 1, 2, 9, 1, 3, 5, 1, 2, 1, 1,
\\ 1, 1, 3, 1, 7, 1, 31, 1, 2, 4, 5

\\ A019434 = 3, 5, 17, 257, 65537

{
  \\ next at k=32, but that overflows in 32-bits

  \\ 0, 1, 1, 2, 9, 1, 3
  my(v=OEIS_data("A019434"),
     f=sum(i=1,#v,1/v[i]),
     k=23,
     e=1/( 2^(2^k) + 1 ),
     t=[f, f+e],
     c=interv_contfrac(t),
     want=OEIS_data("A173995"));
  print(#c);
  print(#want);
  check_equal(want[1..#c], c);

  \\ [0, 1, 1, 2, 9, 1, 3, 5, 1, 2, 1, 1, 1, 1, 3, 1, 7, 1, 31, 1, 2, 4, 5]
}



\\-----------------------------------------------------------------------------
\\ A352677 largest same value binary and base phi
\\
\\ digits 11.xxx  is (2-phi) + (1-1) above
\\ radix point

my(phi=quadgen(5)); \
A352677_vector(len) =
{
  my(r=2-phi,b=1,p=1);
  vector(len,i,
         if(i<3,1, b/=2;p/=phi;
            my(new=r-p+b); if(new>=0,r=new;1,0)));
}
{
  my(want=[1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0]);
  check_equal(A352677_vector(#want), want);
  check_equal(vecsum(A352677_vector(1000)), 276);
}
\\ print(A352677_vector(137));quit


\\-----------------------------------------------------------------------------
\\ sqrt(phi)
\\
\\ A352501  lower
\\    11, 386, 2441, 25748423, 637519684,
\\ A352502  higher
\\    14, 491, 3105, 32752500, 810937565,

contfrac_to_rational(v) = \
  my(m=contfracpnqn(v)); m[1,1]/m[2,1];

\\ digits of sqrt(s) in base phi
sqrt_base_phi_digits(s,len) =
{
  \\ t^2 <= s
  my(t=0, k=0);
  while(phi^k<s,k++);
  vector(len,i, k--;
         if((t+phi^k)^2 <= s,
            t += phi^k; 1,
            0));
}

sqrt_phi_base_phi_digits(len) =
{
  \\ t^2 <= s
  \\ (T/phi^k)^2 <= S/phi^k
  \\ my(r=0,s=1/phi);
  \\ vector(len,i,
  \\        t*=phi; s*=phi^2;
  \\        if((t+1)^2 <= s, t++;1, 0));

  \\ my(t=0,s=phi);
  \\ vector(len,i, my(d=(t+1)^2<=s);
  \\        t+=d;t*=phi; s*=phi^2; d);

  my(t=0,s=phi);
  vector(len,i, my(d=(t+1)^2<=s);
         t=(t+d)*phi; s*=phi^2; d);
}
\\ sqrt_phi_base_phi_digits(15)

{
  my(t=interv_solve_bisection([1,2], x->x^4-x^2-1, 1000));
  \\ sqrt(phi)
  \\ polroots(x^4-x^2-1)
  \\ print(t*1.0);

  \\ continued fraction terms
  my(c=interv_contfrac(t));
  my(want=OEIS_data("A331692"));
  \\ print(c);
  \\ print(want);
  check_equal(c[1..#want],want,
              "A331692 sqrt(phi) continued fraction");

  my(want_lower=[
    11, 386, 2441, 25748423, 637519684,
    2799936925, 3934324789543,
    127501370029150, 21274660147684109,
    644571595359295797,
    15845190736671957299,
    995980378496501932493,
    47375682236837399943653,
    213688560255016550712685,
    28372206851301867342910959 ]);

  \\ A225204 convergents numreators
  \\ 1, 4, 5, 14, 159, 491, 2614, 3105, 33664
  my(A225204(n) =
     numerator(contfrac_to_rational(c[1..n])));
  my(A225205(n) =
     denominator(contfrac_to_rational(c[1..n])));

  OEIS_check_func("A225204",A225204,['offset,1]);
  OEIS_check_func("A225205",A225205,['offset,1]);

  \\ print(select(n->issquare(A225204(n)) && issquare(A225205(n)), [1..100])); quit;

  if(0,
     read("various-Fibbinary.gp");
     my(got_indices=select(n->
       A225204(n)^2 ==
       Zeckendorf_shift_up((A225205(n)^2)),
               [1..#c]));
     print(got_indices);
     \\ not in OEIS: 4, 6, 8, 16, 20, 21, 26, 28, 35, 38, 41, 45, 48

     my(got_lower=apply(A225205,got_indices));
     check_equal(got_lower[1..#want_lower],
                 want_lower);

     for(n=1,20,
        my(f=contfrac_to_rational(c[1..n]),
           num=numerator(f),
           den=denominator(f));
        print(f);
        if(setsearch(want_lower,denominator(f)),
           print("  ***");
           print("  "to_Zeckendorf(num^2));
           print("  "to_Zeckendorf(den^2));
          );
        print();
     );
     quit);

  \\ print(interv_Engel_terms(t,15));quit
  \\ not in OEIS: 1, 4, 12, 18, 41, 42, 94, 162, 252

  \\ A139339 decimal digits sqrt(phi)
  my(want=OEIS_data("A139339"),
     got=interv_frac_digits(t,,#want));
  check_equal(got,want);

  \\ A352594 sqrt(phi) greedy base phi
  my(want=OEIS_data("A352594"),
     got=interv_frac_digits(t,phi,#want));
  check_equal(got,want);

  check_equal(sqrt_base_phi_digits(phi,#want),
              want);
  check_equal(sqrt_phi_base_phi_digits(#want),
              want);

  my(phi=quadgen(5));
  my(a_vector(len) = my(t=0,s=phi); vector(len,i, my(d=(t+1)^2<=s); t=(t+d)*phi; s*=phi^2; d);
     );
  check_equal(a_vector(#want), want);
}

\\ r = c + 1/r'
\\ r' = 1/(r-c) = 1/frac(r)

\\ phi = (sqrt5+1)/2     x^2-x-1
\\ s = sqrt(phi)         x^4-x^2-1
\\ floor(s) = sqrt( (sqrt5+1)/2 )
\\          = sqrt( 2*sqrt5 + 2 ) / 2
\\ phi = (c^2+1)/2
\\ r = (f*sqrt(sqrt5 + 1) + x)/d

\\ r = f/g*sqrt(phi) + x/y
\\   = sqrt( (sqrt(5*f^4/g^4) + f^2/g^2*)/2 ) + x/y
\\   = sqrt( 2*(sqrt(5*f^4/g^4) + f^2/g^2*) )/2 + x/y
\\   = (sqrt( 2*y^2*(sqrt(5*f^4/g^4) + f^2/g^2*) ) + x)/(2*y)

\\   =
\\ floor(r)
\\ 1/r = f/g*sqrt(phi)-x/y

{
  my(sqrt5=quadgen(20),
     phi=(1+sqrt5)/2);
  check_equal(x + y*phi, x+y/2 + y/2*sqrt5);
  check_equal(1/(x + y*phi) ==
              (x+y/2 - y/2*sqrt5)
              / ((x+y/2)^2 - y^2/4*5),  1);

  check_equal(1/(x + y*phi) ==
              (4*x+2*y - 2*y*sqrt5)
              / ((2*x+y)^2 - 5*y^2),  1);

  check_equal(sqrt5, 2*phi-1);
  check_equal(1/(x + y*phi) ==
              (4*x + 4*y - 4*y*phi)
              / ((2*x+y)^2 - 5*y^2),  1);
}
\\ phi=quadgen(5)
\\ 1/(x + y*phi)


\\-----------------------------------------------------------------------------
\\ Sqrt 6

{
  \\ sqrt6 as quad
  my(t=[quadgen(24),quadgen(24)]);
  check_equal(t[1]^2 == 6, 1, "sqrt6 quad");

  \\ decimal digits
  my(want=OEIS_data("A010464"),
     got=interv_frac_digits(t,,#want));
  check_equal(got,want);

  \\ continued fration
  my(want=OEIS_data("A040003"),
     got=interv_contfrac(t,#want));
  check_equal(got,want);

  \\ A?????? Engel expansion of sqrt(6)
  \\ not in OEIS: 3, 23, 23, 49, 80, 121, 124
  \\ my(want=OEIS_data("A123456"));
  \\ check_equal(interv_Engel_terms(t,#want),
  \\             want,
  \\             "interv_Engel_terms() of sqrt6");

}

\\-----------------------------------------------------------------------------
\\ Sqrt 2

{
  \\ sqrt2 as quad

  my(sqrt2=quadgen(8),
     t=[sqrt2,sqrt2]);
  check_equal(sqrt2^2 == 2, 1, "sqrt2 quad");

  \\ A028254 Engel expansion of sqrt(2)
  my(want=OEIS_data("A028254"));
  check_equal(interv_Engel_terms(t,#want),
              want,
              "interv_Engel_terms() of sqrt2");

  \\ A006487 greedy Egyptian fractions sqrt(2)
  \\   1, 3, 13, 253, 218201, 61323543802,
  my(want=OEIS_data("A006487"));
  check_equal(interv_Egyptian_terms(t,#want),
              want,
              "interv_Egyptian_terms() sqrt2");
  my(t=interv_from_Egyptian(want));
  check_equal(t[1]^2 < 2, 1,
             "interv_from_Egyptian() sqrt2 surround");
  check_equal(t[2]^2 > 2, 1);

  foreach([["A004539",2],
           ["A004540",3],
           ["A004541",4],
           ["A004542",5],
           ["A004543",6],
           ["A004544",7],
           ["A004545",8],
           ["A004546",9],
           ["A002193",10]], elem,
     my(anum,base); [anum,base] = elem;
     my(want=OEIS_data(anum));
     check_equal(interv_frac_digits(t,base,#want),
                 want,
                 Str("digits sqrt2 base="base)));
  \\ hexadecimal
  \\ not in OEIS: 0, 9, 14, 6, 6, 7, 15, 3

  \\-----------------
  \\ A069139 greedy Egyptian fractions 1/sqrt(2)
  \\   2, 5, 141, 68575, 32089377154,
  my(want=OEIS_data("A069139"));
  check_equal(interv_Egyptian_terms(t/2,#want),
              want,
              "interv_Egyptian_terms() greedy 1/sqrt2");
}

{
  my(t=interv_solve_bisection([1,2], x->x^2 - 2, 1000));

  \\ \\ sqrt2 in base phi
  \\ my(phi=quadgen(5));
  \\ my(want=OEIS_data("A123456"),
  \\    got=interv_frac_digits(t,phi,#want));
  \\ check_equal(got,want);
}


\\-----------------------------------------------------------------------------
\\ A336686 decimal Sum_{k>=0} 1/(k!)!
\\ A336810 continued fraction
\\ moderately wild terms in contrfrac

{
  my(want=OEIS_data("A336686"),
     limit = 6,
     x = sum(k=0,limit, 1/(k!)!),
     t=[x,x+10*1/(limit!)!]);
  check_equal(interv_frac_digits(t,,#want), want);

  my(want=OEIS_data("A336810"));
  check_equal(interv_contfrac(t,#want), want);
  my(want_t=interv_from_contfrac(want));
  check_equal(interv_contains(want_t,t), 1);
}


\\-----------------------------------------------------------------------------
\\ A249270 decimal sum 1/(product of primes <= k)
\\ A346801 Engel expansion

\\ Return an interval for sum 1/(prod primes < k).
\\ FIXME: The upper bound is not very good.
t_sum_prime_product_recips(k) =
{
  my(d=1,ret=0);
  for(i=0,k-1,
     if(isprime(i),d/=i);
     \\ print(d);
     ret+=d);
  [ret,ret+10*d];
}

{
  my(want = OEIS_data("A346801"),
     t = t_sum_prime_product_recips(500));
  check_equal(interv_Engel_terms(t,#want),
              want);

  \\ decimal digits
  my(want=OEIS_data("A249270"),
     want_t=interv_frac_fromdigits(want));
  check_equal(interv_contains(want_t,t), 1);
}

\\-----------------------------------------------------------------------------
\\ Champernowne Decimals

{
  \\ A077771 decimal of
  \\ A077772 contfrac of
  \\ ternary 1, 2, 1,0, 1,1, 1,2, etc all ternary numbers

  my(want=OEIS_data("A077771"),
     champ=concat(vector(500,n,digits(n,3))),
     t=interv_frac_fromdigits(champ,3,0),
     got=interv_frac_digits(t,,#want+1));
  check_equal(got,concat(0,want), "A077771 decimals");

  my(want=OEIS_data("A077772"),  \\ contfrac
     got=interv_contfrac(t,#want));
  check_equal(got,want, "A072117 contfrac");
  my(want_t=interv_from_contfrac(want));
  check_equal(interv_contains(want_t,t), 1);

  my(v=[1, 2, 1,0, 1,1, 1,2]);
  check_equal(fromdigits(v,3), 3929);
  \\ print(fromdigits(v,3)/3.^#v); quit;
  \\ 0.5988416399 per A077771
}


\\------------------------------------------------------------------------------
\\ plastic constant root of x^3-x-1
\\ A060006 decimal digits
\\ A072117 contfrac

{
  my(want=OEIS_data("A060006"),
     t=interv_solve_bisection([1,2], x->x^3-x-1, 500),
     got=interv_frac_digits(t,,#want));
  check_equal(got,want, "A060006 decimals");

  my(want=OEIS_data("A072117"),  \\ contfrac
     got=interv_contfrac(t,#want));
  check_equal(got,want, "A072117 contfrac");
  my(want_t=interv_from_contfrac(want));
  check_equal(interv_contains(want_t,t), 1);
}


\\------------------------------------------------------------------------------
\\ e contfrac = A003417

A003417(n) =
{
  n>=1 || error();
  if(n==1,2, n%3, 1, 2*n\3);
}
{
  OEIS_check_func("A003417",,['offset,1]);
  check_equal(vector(100,n,n+=7; A003417(n)),
              vector(100,n,n+=7; 2*A003417(n-3) - A003417(n-6)));
}
{
  my(t = interv_exp([1,1],50),  \\ e = exp(1)
     got=interv_contfrac(t));
  check_equal(#got, 62);
  check_equal(got,
              vector(#got,n, A003417(n)),
              "interv_contfrac() of e");
}


\\------------------------------------------------------------------------------
\\ A181313 contfrac of sum 1/2^Fibonacci(k) for k>=1
{
  my(v=vector(1000));
  for(k=1,oo,
     my(f=fibonacci(k),
        i=f+1);
     if(i>#v,break);
     v[i]++);
  my(t=interv_frac_fromdigits(v,2),
     got=interv_contfrac(t),
     want=OEIS_data("A181313"));
  \\ print(t*1.0);
  got=got[1..#want];
  check_equal(got,want,
             "A181313 contfrac");
}

\\ A006518  contfrac for k >= 2
{
  my(v=vector(1000));
  for(k=2,oo,
     my(f=fibonacci(k),
        i=f+1);
     if(i>#v,break);
     v[i]++);
  my(t=interv_frac_fromdigits(v,2),
     got=interv_contfrac(t),
     want=OEIS_data("A006518"));
  got=got[1..#want];
  check_equal(got,want,
             "A006518 contfrac");

  t=interv_add_constant(t,1/2);
  \\ print(t*1.0);
  my(want=OEIS_data("A084119"));
  got=interv_frac_digits(t,,#want);
  check_equal(got,want,
             "A006518 decimals");
}


\\------------------------------------------------------------------------------
\\ A082665
progress("A082665");
{
  my(samples=OEIS_data("A082665"),
     t_samples = interv_frac_fromdigits(samples),
     n_max = 100,
     L,
     s = sum(n=3,n_max, L = 1/2^floor(n^log(log(n)))),
     t_sum = [s, s+L]);
  print(t_samples[1]*1.0);
  print(t_sum[1]*1.0);
  check_equal(interv_contains(t_samples,t_sum), 1);
}


\\-----------------------------------------------------------------------------
\\ Sum 1/2^Fibonacci(n)
progress("A084119");

sum_of_fib_to_k(k,start=0) =
{
  my(s=sum(i=start,k, 1/2^fibonacci(i)));
  [s, s + 1/2^fibonacci(k)];
}
sum_of_fib_k_of_bits(bits) = ceil(log(bits)/log(phi) + 5);

{
  my(samples=OEIS_data("A084119"),
     samples_bits = ceil(#samples * log2(10)),
     k = sum_of_fib_k_of_bits(samples_bits));
  print(k);
  my(t_samples = interv_frac_fromdigits(samples),
     t_sum = sum_of_fib_to_k(k,1));
  print(t_samples[1]*1.0);
  print(t_sum[1]*1.0);
  check_equal(interv_contains(t_samples,t_sum), 1);
}
{
  \\ A084119 decimal digits of sum 1/2^Fibonacci
  my(samples=OEIS_data("A084119",'bfile),
     samples_bits = ceil(#samples * log2(10)),
     k = sum_of_fib_k_of_bits(samples_bits),
     t_samples = interv_frac_fromdigits(samples),
     t_sum = sum_of_fib_to_k(k,1));
  print(samples_bits" "k);
  print(t_samples[1]*1.0);
  print(t_sum[1]*1.0);
  check_equal(interv_contains(t_samples,t_sum), 1,
             "A084119 1/2^Fibonacci(n) decimals");
}


\\-----------------------------------------------------------------------------
\\ A100322 Pi^k frac digits n

{
  my(Pi_digits=OEIS_data("A000796",'bfile)[1..300],
     t_Pi=interv_frac_fromdigits(Pi_digits));

  my(a_vector(len) =
     my(ret=vector(len),
        nlenmax=logint(len,10)+1,
        t_pow=[1,1],
        k=0);
     while(hammingweight(ret) < #ret,
           k++;
           t_pow = interv_mul(t_pow, t_Pi);
           my(int=floor(t_pow));
           int[1]==int[2] || error("k="k" not enough precision");
           my(t=interv_frac(t_pow));
           for(nlen=1,nlenmax,
              my(n=interv_floor_to_constant(t*10^nlen););
              if(n==0,break); \\ no initial 0 digit
              if(n<=#ret && !ret[n],
                 print("found k="k" has n="n);
                 ret[n]=k)));
     ret);

  my(want=OEIS_data("A100322"));
  if(1, \\ shorten
     want=want[1..5]);
  my(got=a_vector(#want));
  \\ my(got=a_vector(75));
  check_equal(got[1..#want], want);

  \\ Pi^k fraction part starts with k
  \\ 1, 4, 75, 9424, 12669, 331783
  if(#got >= 75,
     my(want=OEIS_data("A100323"));
     check_equal(select(n->got[n]==n,[1..#got]),
                 want[1..3]));
}
{
  \\ 1, 4, 75, 9424, 12669, 331783
  if(0,  \\ fairly slow
     my(Pi_digits=OEIS_data("A000796",'bfile)[1..5000],
        t_Pi=interv_frac_fromdigits(Pi_digits),
        t=interv_pow_constant(t_Pi,9424));
     t=interv_frac(t);
     check_equal(interv_floor(t*10000), 9424));
}


\\------------------------------------------------------------------------------
check_equal(a,'a);
check_equal(x,'x);
check_equal(want,'want);

progress("end");